@@ -28,6 +28,7 @@ import java.net.InetSocketAddress;
 import java.net.NetworkInterface;
 import java.net.SocketException;
 import java.net.URI;
+import java.net.URISyntaxException;
 import java.net.UnknownHostException;
 import java.nio.ByteBuffer;
 import java.security.PrivilegedAction;
@@ -176,8 +177,20 @@ public final class Inet {
      * @return the host name, or {@code null} if the address has no host name and is unresolved
      */
     public static String getHostNameIfResolved(InetAddress inetAddress) {
-        final String hostString = new InetSocketAddress(inetAddress, 0).getHostString();
-        final String toString = inetAddress.toString();
+        Assert.checkNotNullParam("inetAddress", inetAddress);
+        return getHostNameIfResolved(new InetSocketAddress(inetAddress, 0));
+    }
+
+    /**
+     * Get the host name of the given address, if it is resolved.  Otherwise, return {@code null}.
+     *
+     * @param socketAddress the socket address to check (must not be {@code null})
+     * @return the host name, or {@code null} if the address has no host name and is unresolved
+     */
+    public static String getHostNameIfResolved(InetSocketAddress socketAddress) {
+        Assert.checkNotNullParam("socketAddress", socketAddress);
+        final String hostString = socketAddress.getHostString();
+        final String toString = socketAddress.toString();
         final int slash = toString.lastIndexOf('/');
         if (slash == 0) {
             // it might be unresolved or it might explicitly be ""
@@ -200,11 +213,10 @@ public final class Inet {
         Assert.checkMinimumParameter("defaultPort", 1, defaultPort);
         Assert.checkMaximumParameter("defaultPort", 65535, defaultPort);
         Assert.checkNotNullParam("addressType", addressType);
-        final String uriHost = uri.getHost();
-        if (uriHost == null) {
+        final InetAddress resolved = getResolvedInetAddress(uri, addressType);
+        if (resolved == null) {
             return null;
         }
-        final InetAddress resolved = getAddressByNameAndType(uriHost, addressType);
         final int uriPort = uri.getPort();
         return uriPort != - 1 ? new InetSocketAddress(resolved, uriPort) : new InetSocketAddress(resolved, defaultPort);
     }
@@ -221,6 +233,37 @@ public final class Inet {
         return getResolved(uri, defaultPort, InetAddress.class);
     }
 
+    /**
+     * Get an Internet address for a URI destination, resolving the host name if necessary.
+     *
+     * @param uri the destination URI
+     * @param <T> the type of the {@code InetAddress} to search for
+     * @return the address, or {@code null} if no authority is present in the URI
+     * @throws UnknownHostException if the URI host was existent but could not be resolved to a valid address
+     */
+    public static <T extends InetAddress> T getResolvedInetAddress(URI uri, Class<T> addressType) throws UnknownHostException {
+        final String uriHost = uri.getHost();
+        if (uriHost == null) {
+            return null;
+        }
+        final int length = uriHost.length();
+        if (length == 0) {
+            return null;
+        }
+        return getAddressByNameAndType(uriHost, addressType);
+    }
+
+    /**
+     * Get an Internet address for a URI destination, resolving the host name if necessary.
+     *
+     * @param uri the destination URI
+     * @return the address, or {@code null} if no authority is present in the URI
+     * @throws UnknownHostException if the URI host was existent but could not be resolved to a valid address
+     */
+    public static InetAddress getResolvedInetAddress(URI uri) throws UnknownHostException {
+        return getResolvedInetAddress(uri, InetAddress.class);
+    }
+
     /**
      * Get a copy of the given socket address, but with a resolved address component.
      *
@@ -908,6 +951,24 @@ public final class Inet {
         return address == null ? 0 : address.getScopeId();
     }
 
+    /**
+     * Extract a base URI from the given scheme and socket address.  The address is not resolved.
+     *
+     * @param scheme the URI scheme
+     * @param socketAddress the host socket address
+     * @param defaultPort the protocol default port, or -1 if there is none
+     * @return the URI instance
+     * @throws URISyntaxException if the URI failed to be constructed for some reason
+     */
+    public static URI getURIFromAddress(String scheme, InetSocketAddress socketAddress, int defaultPort) throws URISyntaxException {
+        String host = getHostNameIfResolved(socketAddress);
+        if (isInet6Address(host)) {
+            host = '[' + host + ']';
+        }
+        final int port = socketAddress.getPort();
+        return new URI(scheme, null, host, port == defaultPort ? -1 : port, null, null, null);
+    }
+
     private static byte parseDecimal(String number) {
         int i = Integer.parseInt(number);
         if (i < 0 || i > 255) {
